@@ -1,5 +1,6 @@
 '''Detector for logical errors in SQL queries.'''
 
+from dataclasses import dataclass
 import difflib
 import re
 import sqlparse
@@ -9,7 +10,7 @@ from typing import Callable
 from .base import BaseDetector, DetectedError
 from ..query import Query
 from ..sql_errors import SqlErrors
-from ..catalog import Catalog
+from ..query import Select, SetOperation
 
 
 class LogicalErrorDetector(BaseDetector):
@@ -36,11 +37,9 @@ class LogicalErrorDetector(BaseDetector):
             self.log_55_substituting_existance_negation_with_less_more_than,
             self.log_56_putting_not_in_front_of_incorrect_in_exists,
             self.log_57_incorrect_comparison_operator_or_value,
-            self.log_58_join_on_incorrect_table,
-            self.log_59_join_when_join_needs_to_be_omitted,
+            self.log_58_log_59_log_62_join_errors,
             self.log_60_join_on_incorrect_column,
             self.log_61_join_with_incorrect_comparison_operator,
-            self.log_62_missing_join,
             self.log_63_improper_nesting_of_expressions,
             self.log_64_improper_nesting_of_subqueries,
             self.log_65_extraneous_quotes,
@@ -172,13 +171,67 @@ class LogicalErrorDetector(BaseDetector):
                     ))
         return results
     
-    # TODO: implement
-    def log_58_join_on_incorrect_table(self) -> list[DetectedError]:
-        return []
-    
-    # TODO: implement
-    def log_59_join_when_join_needs_to_be_omitted(self) -> list[DetectedError]:
-        return []
+    def log_58_log_59_log_62_join_errors(self) -> list[DetectedError]:
+        '''
+            Detects join-related errors by comparing the tables used in the proposed query
+            against those in the correct solutions.
+
+            This function identifies three types of join errors:
+            1. Missing Join: A required table is not included in the proposed query.
+            2. Extraneous Join: An unnecessary table is included in the proposed query.
+            3. Incorrect Join: A table is included, but it is not the correct one needed for the join.
+        '''
+                
+        if not self.solutions:
+            return []
+
+        @dataclass(frozen=True)
+        class TableCol:
+            table: str
+            column: str
+
+        results: list[DetectedError] = []
+
+        expected_tables: list[set[TableCol]] = []
+        actual_tables: set[TableCol] = set()
+
+        # Compute expected tables from solutions
+        # NOTE: We expect each solution to use the same set of tables, but we compute
+        #       them separately to handle any discrepancies.
+        for solution in self.solutions:
+            solution_tables: set[TableCol] = set()
+
+            for select in solution.selects:
+                for table in select.referenced_tables:
+                    if table.cte_idx is not None:
+                        continue
+                    solution_tables.add(TableCol(table.schema_name, table.real_name))
+
+            expected_tables.append(solution_tables)
+
+        # Compute actual tables from the proposed query
+        for select in self.query.selects:
+            for table in select.referenced_tables:
+                if table.cte_idx is not None:
+                    continue
+                actual_tables.add(TableCol(table.schema_name, table.real_name))
+
+        # Check for missing joins (expected tables not in actual)
+        common_expected_tables = expected_tables[0].intersection(*expected_tables[1:])
+        all_expected_tables = expected_tables[0].union(*expected_tables[1:])
+
+        if len(actual_tables) < len(common_expected_tables):
+            for missing_table in common_expected_tables - actual_tables:
+                results.append(DetectedError(SqlErrors.LOG_62_MISSING_JOIN, (missing_table.table, missing_table.column)))
+        elif len(actual_tables) > len(all_expected_tables):
+            for extra_table in actual_tables - all_expected_tables:
+                results.append(DetectedError(SqlErrors.LOG_59_JOIN_WHEN_JOIN_NEEDS_TO_BE_OMITTED, (extra_table.table, extra_table.column)))
+        else:
+            for wrong_table in actual_tables - all_expected_tables:
+                results.append(DetectedError(SqlErrors.LOG_58_JOIN_ON_INCORRECT_TABLE, (wrong_table.table, wrong_table.column)))
+
+        return results
+
     
     # TODO: implement
     def log_60_join_on_incorrect_column(self) -> list[DetectedError]:
@@ -188,10 +241,6 @@ class LogicalErrorDetector(BaseDetector):
     def log_61_join_with_incorrect_comparison_operator(self) -> list[DetectedError]:
         return []
 
-    # TODO: implement
-    def log_62_missing_join(self) -> list[DetectedError]:
-        return []
-
     # TODO: implement
     def log_63_improper_nesting_of_expressions(self) -> list[DetectedError]:
         return []
@@ -313,93 +362,97 @@ class LogicalErrorDetector(BaseDetector):
     def log_69_expression_on_incorrect_clause(self) -> list[DetectedError]:
         return []
 
-    # TODO: refactor
     def log_70_extraneous_column_in_select(self) -> list[DetectedError]:
         '''
-        Flags when an extraneous column is included in the SELECT clause,
-        with a special check for inappropriate use of 'SELECT *'.
+        Flags when an extraneous column is included in the SELECT clause.
         '''
-        return []
-    
-        results = []
-        if not self.q_ast or not self.s_ast:
-            return results
 
-        # First, check specifically for the misuse of 'SELECT *'
-        user_selects_star = self._selects_star(self.q_ast)
-        solution_selects_star = self._selects_star(self.s_ast)
+        results: list[DetectedError] = []
 
-        if user_selects_star and not solution_selects_star:
-            results.append((
-                SqlErrors.LOG_70_EXTRANEOUS_COLUMN_IN_SELECT,
-                "Using 'SELECT *' is not correct for this query. Please select only the required columns."
-            ))
-            return results
+        # First, check if the number of columns exceeds the maximum required by any solution
+        column_number_required_max = max(len(sol.main_query.output.columns) for sol in self.solutions)
+        column_number_provided = len(self.query.main_query.output.columns)
 
-        # If the user correctly used 'SELECT *' (because the solution also did),
-        # then there's no need to check for individual extraneous columns.
-        if user_selects_star:
-            return results
+        if column_number_provided > column_number_required_max:
+            results.append(DetectedError(SqlErrors.LOG_70_EXTRANEOUS_COLUMN_IN_SELECT, (column_number_provided, column_number_required_max)))
 
-        # If 'SELECT *' was not misused, proceed with the original logic for named columns.
-        q_cols = self._get_select_columns(self.q_ast)
-        s_cols = self._get_select_columns(self.s_ast)
+        # Then, check for specific extraneous columns
+        columns_required = set.union(*[sol.output_columns_source for sol in self.solutions])
+        columns_provided = self.query.output_columns_source
+        extraneous_columns = columns_provided - columns_required
 
-        q_cols_set = {col.lower() for col in q_cols}
-        s_cols_set = {col.lower() for col in s_cols}
+        for schema, table, column in extraneous_columns:
+            results.append(DetectedError(SqlErrors.LOG_70_EXTRANEOUS_COLUMN_IN_SELECT, (schema, table, column)))
 
-        extraneous_columns = q_cols_set - s_cols_set
-
-        for col_lower in extraneous_columns:
-            # Find the original case from the query for the error message
-            original_col = next((col for col in q_cols if col.lower() == col_lower), col_lower)
-            results.append((
-                SqlErrors.LOG_70_EXTRANEOUS_COLUMN_IN_SELECT,
-                f"The column '{original_col}' is extraneous and should be removed from the SELECT clause."
-            ))
-            
         return results
-
-    # TODO: refactor
+    
     def log_71_missing_column_from_select(self) -> list[DetectedError]:
         '''
-        Flags when a required column is missing from the SELECT clause,
-        correctly handling cases where duplicate columns are required.
+        Flags when a required column is missing from the SELECT clause.
         '''
-        return []
-    
-        results = []
-        if not self.q_ast or not self.s_ast:
-            return results
 
-        q_cols = self._get_select_columns(self.q_ast)
-        s_cols = self._get_select_columns(self.s_ast)
+        results: list[DetectedError] = []
 
-        # Use collections.Counter to track the frequency of each column
-        q_counts = Counter(col.lower() for col in q_cols)
-        s_counts = Counter(col.lower() for col in s_cols)
+        # First, check if the number of columns is less than the minimum required by any solution
+        column_number_required_min = min(len(sol.main_query.output.columns) for sol in self.solutions)
+        column_number_provided = len(self.query.main_query.output.columns)
 
-        # Subtracting counters finds the exact number of missing instances for each column
-        missing_counts = s_counts - q_counts
+        if column_number_provided < column_number_required_min:
+            results.append(DetectedError(SqlErrors.LOG_71_MISSING_COLUMN_FROM_SELECT, (column_number_provided, column_number_required_min)))
+
+        # Then, check for specific missing columns
+        columns_required = set.union(*[sol.output_columns_source for sol in self.solutions])
+        columns_provided = self.query.output_columns_source
+        missing_columns = columns_required - columns_provided
+
+        for schema, table, column in missing_columns:
+            results.append(DetectedError(SqlErrors.LOG_71_MISSING_COLUMN_FROM_SELECT, (schema, table, column)))
 
-        # Iterate through the elements of the resulting counter to generate a message for each missing column
-        for col_lower in missing_counts.elements():
-            # Find the original case from the solution for a more user-friendly message
-            original_col = next((col for col in s_cols if col.lower() == col_lower), col_lower)
-            results.append((
-                SqlErrors.LOG_71_MISSING_COLUMN_FROM_SELECT,
-                f"A required column '{original_col}' is missing from the SELECT clause."
-            ))
-                
         return results
     
-    # TODO: implement
     def log_72_missing_distinct_from_select(self) -> list[DetectedError]:
+        '''Flags when DISTINCT is missing from a SELECT that requires it.'''
+
+        def _is_distinct(so: SetOperation) -> bool:
+            output = so.output
+            columns = len(output.columns)
+            longest_constraint = max(len(c.columns) for c in output.unique_constraints) if output.unique_constraints else 0
+
+            return longest_constraint >= columns
+
+        # ensure all solutions are DISTINCT
+        requires_distinct = all(_is_distinct(sol.main_query) for sol in self.solutions)
+
+        # At least one solution doesn't require DISTINCT, so it's not necessary for the query
+        # Skip this check
+        if not requires_distinct:
+            return []
+        
+        if not _is_distinct(self.query.main_query):
+            return [DetectedError(SqlErrors.LOG_72_MISSING_DISTINCT_FROM_SELECT)]
+        
         return []
 
-    # TODO: implement
     def log_73_missing_as_from_select(self) -> list[DetectedError]:
-        return []
+        results: list[DetectedError] = []
+
+        # ensure we have the correct columns in both amount and source
+        extraneous_columns = self.log_70_extraneous_column_in_select()
+        missing_columns = self.log_71_missing_column_from_select()
+
+        if extraneous_columns or missing_columns:
+            return results  # skip AS check if column count is already wrong
+
+        expected_aliases: set[str] = set.intersection(*[set(col.name for col in sol.main_query.output.columns) for sol in self.solutions])
+        provided_aliases: set[str] = set(col.name for col in self.query.main_query.output.columns)
+
+        missing_aliases = expected_aliases - provided_aliases
+
+        for alias in missing_aliases:
+            results.append(DetectedError(SqlErrors.LOG_73_MISSING_AS_FROM_SELECT, (alias,)))
+
+        return results
+
 
     # TODO: refactor
     def log_74_missing_column_from_order_by(self) -> list[DetectedError]:
